# -*- coding: utf-8 -*-
"""
Created on Thu Sep 21 11:57:41 2017

@author: ppradeep

----------------------------------------------------------------------------------------
This script runs the best model for POD development on n (=100) bootstrap samples to 
obtain 100 final models which can be used for prediction on new chemicals
Inputs: 
1. Data file with effect level data per chemical
2. PubChem fingerprints
3. CDK descriptors
4. PADel descriptors
Outputs:
1. Descriptor normalization vector (pickle files)
2. Final 100 models (pickled files)
----------------------------------------------------------------------------------------
"""
#%%
## Import packages
import warnings
warnings.filterwarnings('ignore')
import time
import random

import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import pickle 
import sklearn.metrics as sm
from sklearn.metrics import r2_score
from sklearn.model_selection import train_test_split


## Import libraries
import pandas as pd
import numpy as np

# Classifiers
from sklearn.ensemble import RandomForestRegressor
# Machine learning relevant 
from sklearn.feature_selection import SelectPercentile, f_classif
from sklearn.feature_selection import RFE
from sklearn.feature_selection import VarianceThreshold 
from sklearn import preprocessing
from sklearn.model_selection import cross_val_predict
from sklearn.model_selection import GridSearchCV

#%%
# Normalize descriptors: Transform variables to mean=0, variance=1
def normalizeDescriptors(X):
    scaler = preprocessing.StandardScaler().fit(X)
    transformed = scaler.transform(X)
    x_norm = pd.DataFrame(transformed, index = X.index) 
    x_norm.columns = X.columns
    return(scaler, x_norm)
    
def distribution_samples(Y, sigma, seed, n):
    Y_bootstrap = Y
    Y_bootstrap.reset_index(inplace = True)
    for idx in Y_bootstrap.index:
        values = np.random.RandomState(seed).normal(Y_bootstrap.loc[idx,'toxval_numeric'], sigma, n)
        for i, val in enumerate(values):
            Y_bootstrap.loc[idx, 'bootstrap_%d'%i] = val
    Y_bootstrap.index = Y_bootstrap['index']
    del Y_bootstrap['index']
    return Y_bootstrap

#%%
####-----------------------------------------------------------------------------------------------------------------
## Set working dorectory
####-----------------------------------------------------------------------------------------------------------------
path = 'Y:/Projects/ToxvalModels/'
path = 'C:/Users/Administrator/OneDrive/Profile/Desktop/ToxvalModels/'

####-----------------------------------------------------------------------------------------------------------------
## Read training data 
####-----------------------------------------------------------------------------------------------------------------
data_file = pd.read_excel(path+'data/toxval_pod_summary_human health_2019-08-20.xlsx', index_col = 'casrn')

#%%
####-----------------------------------------------------------------------------------------------------------------
## Read training fingerprints
####-----------------------------------------------------------------------------------------------------------------
# PubChem fingerprints
pubchem = pd.read_csv(path+'data/toxval_pubchem.csv', index_col = 'row ID')
# PaDEL descriptors
padel = pd.read_csv(path+'data/toxval_padel.csv', index_col = 'Name').dropna()
# CDK descriptors
cdk = pd.read_csv(path+'data/toxval_cdk.csv', index_col = 'row ID').dropna()

#%%
####-----------------------------------------------------------------------------------------------------------------
## Normalize the descriptors and Save the normalization vector 
####-----------------------------------------------------------------------------------------------------------------
padel_scaler, padel = normalizeDescriptors(padel)
cdk_scaler, cdk = normalizeDescriptors(cdk)
pickle.dump(padel_scaler, open(path+'finalmodels/padel_scaler.sav', 'wb'))
pickle.dump(cdk_scaler, open(path+'finalmodels/cdk_scaler.sav', 'wb'))

#%%
####-----------------------------------------------------------------------------------------------------------------
## Load features for the model and set the X vector for model building
####-----------------------------------------------------------------------------------------------------------------
features = pd.read_csv(path+'output/FinalModelFeatures.csv')

# PubChem
retain = [str(val.replace("'", "").replace(" ", "")) for val in features.ix[0,'Fingerprints'].split(',')]
retain[0] = retain[0].replace("[", "")
retain[len(retain)-1] = retain[len(retain)-1].replace("]","") ##manually check the last entry and correct it
pubchem_model = pubchem.loc[:,retain]

# PaDEL
retain = [str(val.replace("'", "").lstrip()) for val in features.ix[0,'PaDEL'].split(',')]
retain[0] = retain[0].replace("[", "")
retain[len(retain)-1] = retain[len(retain)-1].replace("]","") ##manually check the last entry and correct it
padel_model = padel.loc[:,retain]

# CDK
retain = [str(val.replace("'", "").lstrip()) for val in features.ix[0,'CDK'].split(',')]
retain[0] = retain[0].replace("[", "")
retain[len(retain)-1] = retain[len(retain)-1].replace("]","") ##manually check the last entry and correct it
cdk_model = cdk[retain]

# Set the X vector 
X = pd.concat([pubchem_model, padel_model, cdk_model], axis = 1).dropna()
#%%
####-----------------------------------------------------------------------------------------------------------------
## Select subsets of data for final models studytype-species combinations
####-----------------------------------------------------------------------------------------------------------------
specie = ['rat', 'mouse', 'rabbit']
study = ['chronic', 'cancer', 'subchronic', 'repeat dose', 'short-term', 'reproductive', 'multigenerational reproductive', 'developmental', 'reproductive developmental', 'subacute']

# Response data, Y, for developing QSAR models #'chronic', 'subchronic', 'reproductive', 'developmental, 'subacute'
Y_all = data_file[(data_file['risk_assessment_class'].isin(study)) \
               & (data_file['toxval_units'] == 'mg/kg-day') \
               & (data_file['toxval_type_supercategory'] == 'Point of Departure') \
               & (~data_file['toxval_type'].isin(['ERCL10','NOTEL', \
                  'EC0', 'EC5', 'EC10', 'EC20', 'EC50', 'EC90', 'EC100',\
                  'IC50', 'IC100'])) \
               & (data_file['species_common'].isin(specie))][['toxval_numeric', 'species_common', 'risk_assessment_class']] 
               
## Convert species to number
Y_all.loc[Y_all['species_common']=='rat', 'specie'] = 0
Y_all.loc[Y_all['species_common']=='mouse', 'specie'] = 1
Y_all.loc[Y_all['species_common']=='rabbit', 'specie'] = 2

## Convert study type to number
Y_all.loc[Y_all['risk_assessment_class']=='chronic', 'study'] = 0
Y_all.loc[Y_all['risk_assessment_class']=='cancer', 'study'] = 0

Y_all.loc[Y_all['risk_assessment_class']=='subchronic', 'study'] = 1
Y_all.loc[Y_all['risk_assessment_class']=='repeat dose', 'study'] = 1
Y_all.loc[Y_all['risk_assessment_class']=='short-term', 'study'] = 1

Y_all.loc[Y_all['risk_assessment_class']=='reproductive', 'study'] = 2
Y_all.loc[Y_all['risk_assessment_class']=='multigenerational reproductive', 'study'] = 2

Y_all.loc[Y_all['risk_assessment_class']=='developmental', 'study'] = 3
Y_all.loc[Y_all['risk_assessment_class']=='reproductive developmental', 'study'] = 3

Y_all.loc[Y_all['risk_assessment_class']=='subacute', 'study'] = 4

## Log transform the values
Y_all['toxval_numeric'] = np.log10(Y_all['toxval_numeric'])

#%%
####-----------------------------------------------------------------------------------------------------------------
## Set the Y vector 
####-----------------------------------------------------------------------------------------------------------------
Y = Y_all.groupby([Y_all.index, 'risk_assessment_class', 'species_common']).median().reset_index()
Y.index = Y.casrn

#%%
####-----------------------------------------------------------------------------------------------------------------
## Set the Data to model
####-----------------------------------------------------------------------------------------------------------------
#------------------
XY = X.join(Y).dropna() # species, r_a_c
del XY['species_common']
del XY['risk_assessment_class']

# Set the X data
X_bootstraps = XY[XY.columns.difference(['toxval_numeric','casrn'])]
#%%
#############################################################################################################
##------------------------------------------- Build Models ------------------------------------------- ----------#
#############################################################################################################
n_bootstraps = 100
# Create bootstrap values for each chemical
XY_bootstraps = distribution_samples(XY[['study', 'specie', 'toxval_numeric']], 0.5, 40, n_bootstraps)

for n in range(0, n_bootstraps):
    # Time at start of each boostrap run
    t0 = time.time()
   
    Y_bootstrap = XY_bootstraps['bootstrap_%d'%n] 

    #Build the model
    clf = RandomForestRegressor(bootstrap=True, criterion='mse', max_depth=None,\
           max_features='sqrt', max_leaf_nodes=None,\
           min_impurity_decrease=0.0, min_impurity_split=None,\
           min_samples_leaf=1, min_samples_split=2,\
           min_weight_fraction_leaf=0.0, n_estimators=750, n_jobs=None,\
           oob_score=False, random_state=5, verbose=0, warm_start=False)

    clf.fit(X_bootstraps,Y_bootstrap)
    pkl_filename = path+'finalmodels/model_bootstrap_%d.pkl'%n
    with open(pkl_filename, 'wb') as file:
        pickle.dump(clf, file)
        
    # Time at end of each boostrap model; time elapsed
    t1 = time.time()
    total_time = t1-t0
    print("Bootstrap %d. Total time elapsed: %f" %(n, total_time))
    
#%%