# -*- coding: utf-8 -*-
"""
Created on Tue Mar 17 12:19:40 2020

@author: ppradeep

----------------------------------------------------------------------------------------
This script runs different machine learning algorithms on the dataset to find the best 
model as validated using a 5-fold cross validation on a 80% training set and 20% test 
set.
Inputs: 
1. Data file with effect level data per chemical
2. PubChem fingerprints
3. CDK descriptors
4. PaDEL descriptors
Outputs:
1. Summary metrics file per model
2. Training and test data distribution plots
3. Observed vs predicted plots
4. Enrichment analysis plots
----------------------------------------------------------------------------------------

"""
#%%
############################################################################################################
# Import libraries
############################################################################################################
import warnings
warnings.filterwarnings('ignore')
import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
import sklearn.metrics as sm
from sklearn.metrics import r2_score
from sklearn.model_selection import train_test_split
# Classifiers
from sklearn.linear_model import LinearRegression
from sklearn.neighbors import KNeighborsRegressor
from sklearn.ensemble import RandomForestRegressor
from sklearn.ensemble import GradientBoostingRegressor 
from sklearn import svm
# Machine learning relevant 
from sklearn.feature_selection import SelectPercentile, f_classif
from sklearn.feature_selection import RFE
from sklearn.feature_selection import VarianceThreshold 
from sklearn import preprocessing
from sklearn.model_selection import cross_val_predict
from sklearn.model_selection import GridSearchCV

#%%
############################################################################################################
# User defined functions
############################################################################################################
def selectFeatures_VarThresh(X, threshold):
    sel = VarianceThreshold(threshold=(threshold * (1 - threshold))) 
    X_sel = sel.fit_transform(X)
    # Convert it into a dataframe 
    x_tr = pd.DataFrame(X_sel, index = X.index) 
    x_tr.columns = X.columns[sel.get_support(indices = True)]
    return x_tr

## Remove culumns with >80% correlation
def correlation(dataset, threshold):
    col_corr = set() # Set of all the names of deleted columns
    corr_matrix = dataset.corr()
    for i in range(len(corr_matrix.columns)):
        for j in range(i):
            if corr_matrix.iloc[i, j] >= threshold:
                colname = corr_matrix.columns[i] # getting the name of column
                col_corr.add(colname)
                if colname in dataset.columns:
                    del dataset[colname] # deleting the column from the dataset
    return dataset

# Normalize descriptors: Transform variables to mean=0, variance=1
def normalizeDescriptors(X):
    x_norm = preprocessing.scale(X)
    x_norm = pd.DataFrame(x_norm, index = X.index) 
    x_norm.columns = X.columns
    return x_norm

# Univariate feature selection with F-test for feature scoring
# We use the default selection function: the 10% most significant features
def selectFeatures_perc(X, Y, percentile):
    model = SelectPercentile(f_classif, percentile)
    model = model.fit(X, Y) #convert datatype for use in the fit function
    scores = -np.log10(model.pvalues_)
    scores /= scores.max()
    X_tr = model.transform(X)
    ## Convert it into a dataframe 
    X_tr = pd.DataFrame(X_tr, index = X.index) 
    X_tr.columns = X.columns[model.get_support(indices=True)] 
    return X_tr

def selectFeatures_RFE(X, Y, n_features_to_select):
    model = LinearRegression()
    rfe = RFE(model, n_features_to_select )
    rfe = rfe.fit(X, Y) #convert datatype for use in the fit function
    X_tr = rfe.transform(X)
    ## Convert it into a dataframe 
    X_tr = pd.DataFrame(X_tr, index = X.index) 
    X_tr.columns = X.columns[rfe.get_support(indices=True)] 
    return X_tr

def returnparams_knn(n_fold, X, Y):
    parameters = {'weights':['uniform', 'distance'], 'n_neighbors':[3,4,5], 'algorithm': ['auto', 'ball_tree', 'kd_tree', 'brute']}
    clf = KNeighborsRegressor()
    grid_search = GridSearchCV(clf, cv = n_fold, param_grid = parameters)
    grid_search.fit(X, Y)
    knn_params = grid_search.best_params_   
    print(knn_params)
    return knn_params

def returnparams_svm(n_fold, X, Y):
    parameters = {'kernel':['linear', 'rbf'], 'C':[0.1, 1, 10], 'gamma':[0.01, 0.1, 1], 'epsilon': [0.1, 1]}
    clf = svm.SVR()
    grid_search = GridSearchCV(clf, cv = 5, param_grid = parameters)
    grid_search.fit(X, Y)
    svm_params = grid_search.best_params_
    print(svm_params)
    return svm_params
    
def returnparams_rf(n_fold, X, Y):
    parameters = {'n_estimators': [250,500, 750, 1000, 1500], 'max_features': ['auto', 'sqrt'], 'random_state':[5]} #'sqrt'
    clf = RandomForestRegressor()
    grid_search = GridSearchCV(clf, cv = n_fold, param_grid = parameters)
    grid_search.fit(X, Y)
    rf_params = grid_search.best_params_
    print(rf_params)
    return rf_params
    
def returnparams_gbr(n_fold, X, Y):
    parameters = {'n_estimators': [1000, 1250], 'max_depth': [2,3,4], \
                  'random_state':[5], 'learning_rate': [0.01, 1], 'loss': ['ls', 'lad']}
    clf = GradientBoostingRegressor()
    grid_search = GridSearchCV(clf, cv = n_fold, param_grid = parameters)
    grid_search.fit(X, Y)
    gbr_params = grid_search.best_params_      
    print(gbr_params)    
    return gbr_params
   
def predict_y(clf, X, Y, n_fold):
    y = cross_val_predict(clf, X = X, y = Y, cv = n_fold)
    return y

def predict_test_y(clf, X, Y, X_test):
    clf = clf.fit(X,Y)
    y = clf.predict(X_test)
    return y
#%%    
############################################################################################################
# Set working directory path
############################################################################################################
path = 'Z:/Projects/ToxvalModels/'
path = 'C:/Users/Administrator/OneDrive/Profile/Desktop/ToxvalModels/'

#%%
############################################################################################################
# Read the data
############################################################################################################
# Read the input data file
data_file = pd.read_excel(path+'data/toxval_pod_summary_human health_2019-08-20.xlsx', index_col = 'casrn')

## Read PubChem fingerprints and CDK descriptors generated in KNIME
pubchem = pd.read_csv(path+'data/toxval_pubchem.csv', index_col = 'row ID')
padel = pd.read_csv(path+'data/toxval_padel.csv', index_col = 'Name').dropna()
cdk = pd.read_csv(path+'data/toxval_cdk.csv', index_col = 'row ID').dropna()


#%%
############################################################################################################
# Select subsets of data based on studytype-species combinations
############################################################################################################
#specie = ['rat']
#specie = ['mouse']
#specie = ['rabbit']
#specie = ['rat', 'mouse']
specie = ['rat', 'mouse', 'rabbit']

#study = ['chronic', 'cancer']
#study = ['subchronic', 'repeat dose', 'short-term']
#study = ['reproductive','multigenerational reproductive']
#study = ['developmental', 'reproductive developmental']
#study = ['subacute']
study = ['chronic', 'cancer', 'subchronic', 'repeat dose', 'short-term', 'reproductive', 'multigenerational reproductive', 'developmental', 'reproductive developmental', 'subacute']

# Response data, Y, for developing QSAR models #'chronic', 'subchronic', 'reproductive', 'developmental, 'subacute'
Y_all = data_file[(data_file['risk_assessment_class'].isin(study)) \
               & (data_file['toxval_units'] == 'mg/kg-day') \
               & (data_file['toxval_type_supercategory'] == 'Point of Departure') \
               & (~data_file['toxval_type'].isin(['ERCL10','NOTEL', \
                  'EC0', 'EC5', 'EC10', 'EC20', 'EC50', 'EC90', 'EC100',\
                  'IC50', 'IC100'])) \
               & (data_file['species_common'].isin(specie))][['toxval_numeric', 'species_common', 'risk_assessment_class']] 
               
## Convert species to number
Y_all.loc[Y_all['species_common']=='rat', 'specie'] = 0
Y_all.loc[Y_all['species_common']=='mouse', 'specie'] = 1
Y_all.loc[Y_all['species_common']=='rabbit', 'specie'] = 2

## Convert study type to number
Y_all.loc[Y_all['risk_assessment_class']=='chronic', 'study'] = 0
Y_all.loc[Y_all['risk_assessment_class']=='cancer', 'study'] = 0

Y_all.loc[Y_all['risk_assessment_class']=='subchronic', 'study'] = 1
Y_all.loc[Y_all['risk_assessment_class']=='repeat dose', 'study'] = 1
Y_all.loc[Y_all['risk_assessment_class']=='short-term', 'study'] = 1

Y_all.loc[Y_all['risk_assessment_class']=='reproductive', 'study'] = 2
Y_all.loc[Y_all['risk_assessment_class']=='multigenerational reproductive', 'study'] = 2

Y_all.loc[Y_all['risk_assessment_class']=='developmental', 'study'] = 3
Y_all.loc[Y_all['risk_assessment_class']=='reproductive developmental', 'study'] = 3

Y_all.loc[Y_all['risk_assessment_class']=='subacute', 'study'] = 4

## Log transform the values
Y_all['toxval_numeric'] = np.log10(Y_all['toxval_numeric'])

# Get a percentage of data by species and study type
Y_all.groupby('specie').count()*100/len(Y_all)
Y_all.groupby('study').count()*100/len(Y_all)

#%%
############################################################################################################
# Data for Model 1 (different subsets of species and study types)
############################################################################################################
## Assign an overall single POD value per chemical across all study types and species - use the median of all values
Y1 = Y_all.groupby(Y_all.index)['toxval_numeric'].median()

#--------------------------------------Feature Selection---------------------------------------------------#
## Fingerprints and Descriptors
# Pubchem
X1_pubchem = pubchem.loc[Y1.index,:].dropna()
# Feature Selection on X1 fingerprints using 80% variance threshold
X1_pubchem = selectFeatures_VarThresh(X1_pubchem, 0.8)
# Remove culumns with >80% correlation
X1_pubchem = correlation(X1_pubchem, 0.80)
# Padel
X1_padel = padel.loc[Y1.index,:].dropna()
## Feature selection from Padel descriptors
X1_padel = normalizeDescriptors(X1_padel)
X1_padel = selectFeatures_RFE(X1_padel, Y1[X1_padel.index], 5)
# CDK
X1_cdk = cdk.loc[Y1.index,:].dropna()
## Feature selection from Padel descriptors
X1_cdk = normalizeDescriptors(X1_cdk)
X1_cdk = selectFeatures_RFE(X1_cdk, Y1[X1_cdk.index], 5)
#------------------------------------------Set the X and Y vector-------------------------------------------------#
X1 = pd.concat([X1_pubchem, X1_padel, X1_cdk], axis = 1).dropna()
# Update Y based on X columns that are retained
Y1 = Y1[X1.index]

#%%
#############################################################################################################
# Data for Model 2 (using species and study types as descriptors to the model)
#############################################################################################################
## Assign a single POD value per chemical within each study types and species combination - use the median of all values
Y2 = Y_all.groupby([Y_all.index, 'risk_assessment_class', 'species_common']).median().reset_index()
Y2.index = Y2.casrn

datatomodel = pd.concat([X1_pubchem, X1_padel, X1_cdk], axis = 1).join(Y2).dropna() # species, r_a_c

X2 = datatomodel[datatomodel.columns.difference(['index','toxval_numeric','risk_assessment_class','casrn', 'species_common'])]
Y2 = datatomodel['toxval_numeric']

#%%
#############################################################################################################
# Set the X and Y train and test vector
#############################################################################################################

## Model 1
#X = X1
#Y = Y1

## Model2
X = X2
Y = Y2

## Divide training and test vector: 80% training and 20% test)
X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size = 0.20, random_state = 40)
# calculate standard deviation of training and test set
sigma_train = np.std(Y_train)
sigma_test = np.std(Y_test)

#%%
#############################################################################################################
# Histogram of FINAL training and test data superimposed on each other
#############################################################################################################
plt.figure(figsize=(8, 6), dpi = 200)
Y_train.hist(label = 'Training (n = %d | $\sigma$ = %0.2f)' %(len(Y_train), sigma_train), alpha = 0.75, color = 'r')
Y_test.hist(label = 'Test (n = %d| $\sigma$ = %0.2f)' %(len(Y_test), sigma_test), alpha = 0.75, color = 'g')
plt.xlabel('Effect Level$_{tr} (log_{10} mg/kg/day)$', size = 24, labelpad = 10)
plt.ylabel('Frequency', size = 24, labelpad = 10)
plt.xticks(fontsize = 24)#, rotation = 90)
plt.yticks(fontsize = 24)
plt.legend(fontsize = 14, loc='upper left') 

#plt.savefig(path+'output/TrainTestDist_%s_%s.png' %(''.join(study), ''.join(specie)), bbox_inches='tight')
plt.savefig(path+'output/TrainTestDist_%s_%s_studytype_species.png' %(''.join(study), ''.join(specie)), bbox_inches='tight')
plt.show()

#%%
#############################################################################################################
# Train the model and calculate errors using n_fold cross validation
#############################################################################################################

n_fold = 5
classifiers = [#KNeighborsRegressor(**returnparams_knn(n_fold, X_train, Y_train)),\
               #svm.SVR(**returnparams_svm(n_fold, X_train, Y_train)),\
               RandomForestRegressor(**returnparams_rf(n_fold, X_train, Y_train)),\
               #GradientBoostingRegressor(**returnparams_gbr(n_fold, X_train, Y_train))     
               ]


# Make a n_fold cross validation prediction for each classifier
metrics = pd.DataFrame()
for clf in classifiers:
    #------------------------Internal 5-fold CV--------------------------------#
    Y_train_predicted = pd.Series(predict_y(clf, X_train, Y_train, n_fold))
    Y_train_predicted.index = Y_train.index
    metrics.loc[str(clf).split('(')[0],'RMSE_train'] = np.sqrt(sm.mean_squared_error(Y_train, Y_train_predicted))
    metrics.loc[str(clf).split('(')[0], 'R2_train'] = r2_score(Y_train, Y_train_predicted)

    ## Summarize predictions on chemical level
    Y_train_predicted_byChem = Y_train_predicted.groupby([Y_train_predicted.index]).median()
    Y_train_byChem = Y_train.groupby([Y_train.index]).median()
    metrics.loc[str(clf).split('(')[0],'RMSEbyChem_train'] = np.sqrt(sm.mean_squared_error(Y_train_byChem, Y_train_predicted_byChem))
    metrics.loc[str(clf).split('(')[0], 'R2byChem_train'] = r2_score(Y_train_byChem, Y_train_predicted_byChem)
    
    # Internal 5-fold CV: Plot true vs predicted for each classifier (Y_train_predicted)
    ax = plt.figure(figsize=(8,8), dpi = 200)
    plt.scatter(Y_train, Y_train_predicted, alpha = 0.75, color = 'r')
    plt.plot([Y_train.min()-1,Y_train.max()+1],[Y_train.min()-sigma_train-1, Y_train.max()-sigma_train+1],'b', linestyle = '--')
    plt.plot([Y_train.min()-1, Y_train.max()-sigma_train+1],[Y_train.min()+sigma_train-1, Y_train.max()+1],'b', label = '$\pm1 \sigma$ error interval', linestyle = '--')
    
    plt.legend(loc='lower right', numpoints = 2, scatterpoints = 1, fontsize = 14)
    
    plt.plot([-2,4], [-2,4], 'k')
    plt.xlim([Y_train.min(), Y_train.max()])
    plt.ylim([Y_train.min(), Y_train.max()])
    plt.xticks(fontsize = 28, rotation = 90)
    plt.yticks(fontsize = 28)
    
    plt.xlim([-2,4])
    plt.ylim([-2,4])
    
    ax.text(0.15, 0.84, r'$RMSE: %.2f$' % metrics.loc[str(clf).split('(')[0],'RMSE_train'], fontsize = 20)
    ax.text(0.15, 0.79, r'$RMSE/\sigma: %.2f$' %(metrics.loc[str(clf).split('(')[0],'RMSE_train']/np.std(Y_train)), fontsize = 20)
    ax.text(0.15, 0.74, r'$R^{2}: %.2f$' % metrics.loc[str(clf).split('(')[0],'R2_train'], fontsize = 20)
    
    plt.xlabel('Observed', size = 28, labelpad = 18)
    plt.ylabel('Predicted', size = 28, labelpad = 18)
    #plt.savefig(path+'/output/Int_TvsP_%s_%s_%s' %(str(clf).split('(')[0], ''.join(study), ''.join(specie)), bbox_inches='tight')
    plt.savefig(path+'/output/Int_TvsP__studytype_species.png', bbox_inches='tight')
    
    # Internal 5-fold CV: Plot true vs predicted for each classifier per chemical (Y_train_predicted_byChem)
    ax = plt.figure(figsize=(8,8), dpi = 200)
    plt.scatter(Y_train_byChem, Y_train_predicted_byChem, alpha = 0.75, color = 'r')
    plt.plot([Y_train.min()-1,Y_train.max()+1],[Y_train.min()-sigma_train-1, Y_train.max()-sigma_train+1],'b', linestyle = '--')
    plt.plot([Y_train.min()-1, Y_train.max()-sigma_train+1],[Y_train.min()+sigma_train-1, Y_train.max()+1],'b', label = '$\pm1 \sigma$ error interval', linestyle = '--')
    
    plt.legend(loc='lower right', numpoints = 2, scatterpoints = 1, fontsize = 14)
    
    plt.plot([-2,4], [-2,4], 'k')
    plt.xlim([Y_train.min(), Y_train.max()])
    plt.ylim([Y_train.min(), Y_train.max()])
    plt.xticks(fontsize = 28, rotation = 90)
    plt.yticks(fontsize = 28)
    
    plt.xlim([-2,4])
    plt.ylim([-2,4])
    
    ax.text(0.15, 0.84, r'$RMSE: %.2f$' % metrics.loc[str(clf).split('(')[0],'RMSEbyChem_train'], fontsize = 20)
    ax.text(0.15, 0.79, r'$RMSE/\sigma: %.2f$' %(metrics.loc[str(clf).split('(')[0],'RMSEbyChem_train']/np.std(Y_train)), fontsize = 20)
    ax.text(0.15, 0.74, r'$R^{2}: %.2f$' % metrics.loc[str(clf).split('(')[0],'R2byChem_train'], fontsize = 20)
    
    plt.xlabel('Observed', size = 28, labelpad = 18)
    plt.ylabel('Predicted', size = 28, labelpad = 18)
    #plt.savefig(path+'/output/Int_TvsP_%s_%s_%s' %(str(clf).split('(')[0], ''.join(study), ''.join(specie)), bbox_inches='tight')
    plt.savefig(path+'/output/Int_TvsP__studytype_species_byChem.png', bbox_inches='tight')
      
    
    #---------------- Fit model on entire training data and make predictions for test set---------------- #
    Y_test_predicted = pd.Series(predict_test_y(clf, X_train, Y_train, X_test))
    Y_test_predicted.index = Y_test.index
    metrics.loc[str(clf).split('(')[0],'RMSE_test'] = np.sqrt(sm.mean_squared_error(Y_test, Y_test_predicted))
    metrics.loc[str(clf).split('(')[0], 'R2_test'] = r2_score(Y_test, Y_test_predicted)
    
    # Summarize predictions on chemical level
    Y_test_predicted_byChem = Y_test_predicted.groupby([Y_test_predicted.index]).median()
    Y_test_byChem = Y_test.groupby([Y_test.index]).median()
    metrics.loc[str(clf).split('(')[0],'RMSEbyChem_test'] = np.sqrt(sm.mean_squared_error(Y_test_byChem, Y_test_predicted_byChem))
    metrics.loc[str(clf).split('(')[0], 'R2byChem_test'] = r2_score(Y_test_byChem, Y_test_predicted_byChem)
    
    # Enrichment analysis (external set)
    Y_test_pred_df = pd.DataFrame(index = X_test.index, columns = ['Prediction'])
    Y_test_pred_df['Prediction'] = Y_test_predicted
    Y_test_pred_df['CAS_Rank'] = Y_test_pred_df['Prediction'].rank(ascending=True)
    
    Y_test_df = pd.DataFrame(index = Y_test.index, columns = ['True'])
    Y_test_df['True'] = Y_test
    Y_test_df['CAS_Rank'] = Y_test_df['True'].rank(ascending=True)
    
    x = range(0,110,10)
    
    # Most potent % chemicals 
    truth_list_5 = set(Y_test_df[Y_test_df['CAS_Rank'] <= 5*len(Y_test_df)/100].index)
    truth_list_10 = set(Y_test_df[Y_test_df['CAS_Rank'] <= 10*len(Y_test_df)/100].index)
    truth_list_20 = set(Y_test_df[Y_test_df['CAS_Rank'] <= 20*len(Y_test_df)/100].index)
    truth_list_50 = set(Y_test_df[Y_test_df['CAS_Rank'] <= 50*len(Y_test_df)/100].index)
    
    y_5 = []; y_10 = []; y_20 = []; y_50 = []
    
    for perc in x:
        pred_list = set(Y_test_pred_df[Y_test_pred_df['CAS_Rank'] <= perc*len(Y_test_df)/100].index)
        try:
            y_5.append(100*len(truth_list_5&pred_list)/len(truth_list_5))
        except:
            pass
        #y_10.append(100*len(truth_list_10&pred_list)/len(truth_list_10))
        y_20.append(100*len(truth_list_20&pred_list)/len(truth_list_20))
        y_50.append(100*len(truth_list_50&pred_list)/len(truth_list_50))
    
    plt.figure(figsize=(12, 8), dpi = 200)
    try:
        plt.plot(x,y_5, marker = 'o', color = 'r', label = '$N=5\%$')
    except:
        pass
    #plt.plot(x,y_10, marker = 'o', color = 'r', label = '$N=10\%$')
    plt.plot(x,y_20, marker = 'o', color = 'g', label = '$N=20\%$')
    plt.plot(x,y_50, marker = 'o', color = 'b', label = '$N=50\%$')   
    plt.xlim([0, 100])
    plt.ylim([0, 100])
    plt.xticks(fontsize = 28)
    plt.yticks(fontsize = 28)
    plt.ylabel('% enrichment of $N\%$ most \n potent chemicals (observed)', size = 26, labelpad = 18)
    plt.xlabel('X% most potent chemicals (predicted)', size = 26, labelpad = 18)
    plt.legend(loc='lower right', fontsize = 22)
    #plt.savefig(path+'/output/EnrichmentPlot_%s_%s_%s.png' %(str(clf).split('(')[0], ''.join(study), ''.join(specie)), bbox_inches='tight')
    plt.savefig(path+'/output/EnrichmentPlot_studytype_species.png', bbox_inches='tight')
    
    # Enrichment analysis (external set) per chemical
    Y_test_pred_df_byChem = pd.DataFrame(index = Y_test_predicted_byChem.index, columns = ['Prediction'])
    Y_test_pred_df_byChem['Prediction'] = Y_test_predicted_byChem
    Y_test_pred_df_byChem['CAS_Rank'] = Y_test_pred_df_byChem['Prediction'].rank(ascending=True)
    
    Y_test_df_byChem = pd.DataFrame(index = Y_test_byChem.index, columns = ['True'])
    Y_test_df_byChem['True'] = Y_test_byChem
    Y_test_df_byChem['CAS_Rank'] = Y_test_df_byChem['True'].rank(ascending=True)
    
    x = range(0,110,10)
    
    # Most potent % chemicals 
    truth_list_5 = set(Y_test_df[Y_test_df_byChem['CAS_Rank'] <= 5*len(Y_test_df_byChem)/100].index)
    truth_list_10 = set(Y_test_df[Y_test_df_byChem['CAS_Rank'] <= 10*len(Y_test_df_byChem)/100].index)
    truth_list_20 = set(Y_test_df[Y_test_df_byChem['CAS_Rank'] <= 20*len(Y_test_df_byChem)/100].index)
    truth_list_50 = set(Y_test_df[Y_test_df_byChem['CAS_Rank'] <= 50*len(Y_test_df_byChem)/100].index)
    
    y_5 = []; y_10 = []; y_20 = []; y_50 = []
    
    for perc in x:
        pred_list = set(Y_test_pred_df_byChem[Y_test_pred_df_byChem['CAS_Rank'] <= perc*len(Y_test_df_byChem)/100].index)
        try:
            y_5.append(100*len(truth_list_5&pred_list)/len(truth_list_5))
        except:
            pass
        #y_10.append(100*len(truth_list_10&pred_list)/len(truth_list_10))
        y_20.append(100*len(truth_list_20&pred_list)/len(truth_list_20))
        y_50.append(100*len(truth_list_50&pred_list)/len(truth_list_50))
    
    plt.figure(figsize=(12, 8), dpi = 200)
    try:
        plt.plot(x,y_5, marker = 'o', color = 'r', label = '$N=5\%$')
    except:
        pass
    #plt.plot(x,y_10, marker = 'o', color = 'r', label = '$N=10\%$')
    plt.plot(x,y_20, marker = 'o', color = 'g', label = '$N=20\%$')
    plt.plot(x,y_50, marker = 'o', color = 'b', label = '$N=50\%$')   
    plt.xlim([0, 100])
    plt.ylim([0, 100])
    plt.xticks(fontsize = 28)
    plt.yticks(fontsize = 28)
    plt.ylabel('% enrichment of $N\%$ most \n potent chemicals (observed)', size = 26, labelpad = 18)
    plt.xlabel('X% most potent chemicals (predicted)', size = 26, labelpad = 18)
    plt.legend(loc='lower right', fontsize = 22)
    #plt.savefig(path+'/output/EnrichmentPlot_%s_%s_%s.png' %(str(clf).split('(')[0], ''.join(study), ''.join(specie)), bbox_inches='tight')
    #plt.savefig(path+'output/EnrichmentPlot_%s_%s_%s_studytype_species_byChem'  %('RF', ''.join(study), ''.join(specie)), bbox_inches='tight')
    plt.savefig(path+'/output/EnrichmentPlot_studytype_species_byChem.png', bbox_inches='tight')
    
    # External set: Plot true vs predicted for each classifier    
    ax = plt.figure(figsize=(8,8), dpi = 200)
    
    plt.scatter(Y_test, Y_test_predicted, alpha = 0.75, color = 'g')
    plt.plot([Y_test.min()-2, Y_test.max()-sigma_test+2],[Y_test.min()+sigma_test-2, Y_test.max()+2],'b', label = '$\pm1 \sigma$ error interval', linestyle = '--')
    plt.plot([Y_test.min()-2,Y_test.max()+2],[Y_test.min()-sigma_test-2, Y_test.max()-sigma_test+2],'b', linestyle = '--')
    
    plt.legend(loc='lower right', numpoints = 2, scatterpoints = 1, fontsize = 14)
    #plt.plot(Y_test, Y_test, 'k')
    plt.plot([-2,4], [-2,4], 'k')
    plt.xlim([Y_test.min(), Y_test.max()])
    plt.ylim([Y_test.min(), Y_test.max()])
    
    plt.xticks(fontsize = 28, rotation = 90)
    plt.yticks(fontsize = 28)
    
    plt.xlim([-2,4])
    plt.ylim([-2,4])
    
    ax.text(0.15, 0.84, r'$RMSE: %.2f$' % metrics.loc[str(clf).split('(')[0],'RMSE_test'], fontsize = 20)
    ax.text(0.15, 0.79, r'$RMSE/\sigma: %.2f$' %(metrics.loc[str(clf).split('(')[0],'RMSE_test']/np.std(Y_test)), fontsize = 20)
    ax.text(0.15, 0.74, r'$R^{2}: %.2f$' % metrics.loc[str(clf).split('(')[0],'R2_test'], fontsize = 20)
    
    plt.xlabel('Observed', size = 28, labelpad = 18)
    plt.ylabel('Predicted', size = 28, labelpad = 18)
    
    #plt.title('Classifier: %s' %(str(clf).split('(')[0]))
    #plt.savefig(path+'/output/Ext_TvsP_%s_%s_%s.png' %(str(clf).split('(')[0], ''.join(study), ''.join(specie)), bbox_inches='tight')
    #plt.savefig(path+'output/Ext_TvsP_%s_%s_%s_studytype_species'  %('RF', ''.join(study).replace(' ', ''), ''.join(specie)), bbox_inches='tight')
    plt.savefig(path+'/output/Ext_TvsP_studytype_species.png', bbox_inches='tight')
    
    # External set: Plot true vs predicted for each classifier oer Chemical  
    ax = plt.figure(figsize=(8,8), dpi = 200)
    
    plt.scatter(Y_test, Y_test_predicted, alpha = 0.75, color = 'g')
    plt.plot([Y_test.min()-2, Y_test.max()-sigma_test+2],[Y_test.min()+sigma_test-2, Y_test.max()+2],'b', label = '$\pm1 \sigma$ error interval', linestyle = '--')
    plt.plot([Y_test.min()-2,Y_test.max()+2],[Y_test.min()-sigma_test-2, Y_test.max()-sigma_test+2],'b', linestyle = '--')
    
    plt.legend(loc='lower right', numpoints = 2, scatterpoints = 1, fontsize = 14)
    #plt.plot(Y_test, Y_test, 'k')
    plt.plot([-2,4], [-2,4], 'k')
    plt.xlim([Y_test.min(), Y_test.max()])
    plt.ylim([Y_test.min(), Y_test.max()])
    
    plt.xticks(fontsize = 28, rotation = 90)
    plt.yticks(fontsize = 28)
    
    plt.xlim([-2,4])
    plt.ylim([-2,4])
    
    
    ax.text(0.15, 0.84, r'$RMSE: %.2f$' % metrics.loc[str(clf).split('(')[0],'RMSEbyChem_test'], fontsize = 20)
    ax.text(0.15, 0.79, r'$RMSE/\sigma: %.2f$' %(metrics.loc[str(clf).split('(')[0],'RMSEbyChem_test']/np.std(Y_test)), fontsize = 20)
    ax.text(0.15, 0.74, r'$R^{2}: %.2f$' % metrics.loc[str(clf).split('(')[0],'R2byChem_test'], fontsize = 20)
    
    plt.xlabel('Observed', size = 28, labelpad = 18)
    plt.ylabel('Predicted', size = 28, labelpad = 18)
    
    #plt.title('Classifier: %s' %(str(clf).split('(')[0]))
    #plt.savefig(path+'/output/Ext_TvsP_%s_%s_%s.png' %(str(clf).split('(')[0], ''.join(study), ''.join(specie)), bbox_inches='tight')
    plt.savefig(path+'/output/Ext_TvsP_studytype_species_byChem.png', bbox_inches='tight')
    
    metrics.loc[str(clf).split('(')[0],'Model'] = str(clf)

metrics.to_csv(path+'output\ModelInfo_%s_%s_%s_studytype_species_byChem.csv'  %('RF', ''.join(study).replace(' ', ''), ''.join(specie)))
#metrics.to_csv(path+'output\ModelInfo_%s_%s_%s_studytype_species_byChem.csv'  %(str(clf).split('(')[0], ''.join(study), ''.join(specie)))

#%%
#############################################################################################################
## Feature Importance using RF Model
#############################################################################################################
clf = clf.fit(X_train, Y_train)
features = X_train.columns
importances = clf.feature_importances_
std = np.std([tree.feature_importances_ for tree in clf.estimators_], axis=0)
indices = np.argsort(importances)[::-1]   
      
# Plot the feature importances of the forest
import matplotlib.pyplot as plt
plt.figure(figsize=(25,8), dpi = 200)
plt.bar(range(X_train.shape[1]), importances[indices], color="r", yerr=std[indices], align="center", alpha = 0.75)
#plt.xticks(range(X.shape[1]), indices)
plt.xticks(range(X_train.shape[1]), [features[i] for i in indices], rotation = 90, fontsize = 16)       
plt.xlim([-1, X_train.shape[1]])
plt.yticks(fontsize = 28)
plt.xlabel('Descriptor', fontsize = 28, labelpad = 10)
plt.ylabel('Descriptor Importance', fontsize = 28, labelpad = 10)
plt.savefig(path+'output/RF_FeatureImp.png', bbox_inches='tight')

#%%