# -*- coding: utf-8 -*-
"""
Created on Thu Sep 21 11:57:41 2017

@author: ppradeep

----------------------------------------------------------------------------------------
This script runs the best model for POD development on n (=100) bootstrap samples to 
obtain a 95% confidence interval around the predictions
Inputs: 
1. Data file with effect level data per chemical
2. PubChem fingerprints
3. CDK descriptors
4. PADel descriptors
Outputs:
1. Summary metrics file per model
2. Training and test data distribution plot
3. Sample Observed vs predicted plot
4. Bootstrap predictions and summary
----------------------------------------------------------------------------------------
"""
#%%
## Import packages
import warnings
warnings.filterwarnings('ignore')
import time
import random

import pandas as pd
import matplotlib.pyplot as plt
import numpy as np
from numpy import mean, absolute 
import pickle 
import sklearn.metrics as sm
from sklearn.metrics import r2_score
from sklearn.model_selection import train_test_split

from functions import *

## Import libraries
import pandas as pd
import numpy as np

# Classifiers
from sklearn.linear_model import LinearRegression
from sklearn.neighbors import KNeighborsRegressor
from sklearn.ensemble import RandomForestRegressor
from sklearn.ensemble import GradientBoostingRegressor 
from sklearn import svm

# Machine learning relevant 
from sklearn.feature_selection import SelectPercentile, f_classif
from sklearn.feature_selection import RFE
from sklearn.feature_selection import VarianceThreshold 
from sklearn import preprocessing
from sklearn.model_selection import cross_val_predict
from sklearn.model_selection import GridSearchCV

#%%
## User-defined functions
def selectFeatures_VarThresh(X, threshold):
    sel = VarianceThreshold(threshold=(threshold * (1 - threshold))) 
    X_sel = sel.fit_transform(X)
    # Convert it into a dataframe 
    x_tr = pd.DataFrame(X_sel, index = X.index) 
    x_tr.columns = X.columns[sel.get_support(indices = True)]
    return x_tr

## Remove culumns with >80% correlation
def correlation(dataset, threshold):
    col_corr = set() # Set of all the names of deleted columns
    corr_matrix = dataset.corr()
    for i in range(len(corr_matrix.columns)):
        for j in range(i):
            if corr_matrix.iloc[i, j] >= threshold:
                colname = corr_matrix.columns[i] # getting the name of column
                col_corr.add(colname)
                if colname in dataset.columns:
                    del dataset[colname] # deleting the column from the dataset
    return dataset

# Normalize descriptors: Transform variables to mean=0, variance=1
def normalizeDescriptors(X):
    x_norm = preprocessing.scale(X)
    x_norm = pd.DataFrame(x_norm, index = X.index) 
    x_norm.columns = X.columns
    return x_norm

# Univariate feature selection with F-test for feature scoring
# We use the default selection function: the 10% most significant features
def selectFeatures_perc(X, Y, percentile):
    model = SelectPercentile(f_classif, percentile)
    model = model.fit(X, Y) #convert datatype for use in the fit function
    scores = -np.log10(model.pvalues_)
    scores /= scores.max()
    X_tr = model.transform(X)
    ## Convert it into a dataframe 
    X_tr = pd.DataFrame(X_tr, index = X.index) 
    X_tr.columns = X.columns[model.get_support(indices=True)] 
    return X_tr

def selectFeatures_RFE(X, Y, n_features_to_select):
    model = LinearRegression()
    rfe = RFE(model, n_features_to_select )
    rfe = rfe.fit(X, Y) #convert datatype for use in the fit function
    X_tr = rfe.transform(X)
    ## Convert it into a dataframe 
    X_tr = pd.DataFrame(X_tr, index = X.index) 
    X_tr.columns = X.columns[rfe.get_support(indices=True)] 
    return X_tr

def distribution_samples(Y, sigma, seed, n):
    Y_bootstrap = Y
    Y_bootstrap.reset_index(inplace = True)
    for idx in Y_bootstrap.index:
        values = np.random.RandomState(seed).normal(Y_bootstrap.loc[idx,'toxval_numeric'], sigma, n)
        for i, val in enumerate(values):
            Y_bootstrap.loc[idx, 'bootstrap_%d'%i] = val
    Y_bootstrap.index = Y_bootstrap['index']
    return Y_bootstrap
    
def returnparams_rf(n_fold, X, Y):
    parameters = {'n_estimators': [250,500, 750, 1000, 1500], 'max_features': ['auto', 'sqrt'], 'random_state':[5]} #'sqrt'
    clf = RandomForestRegressor()
    grid_search = GridSearchCV(clf, cv = n_fold, param_grid = parameters)
    grid_search.fit(X, Y)
    rf_params = grid_search.best_params_
    print(rf_params)
    return rf_params
   
def predict_y(clf, X, Y, n_fold):
    y = cross_val_predict(clf, X = X, y = Y, cv = n_fold)
    return y

def predict_test_y(clf, X, Y, X_test):
    clf = clf.fit(X,Y)
    y = clf.predict(X_test)
    return y

#%%
path = 'Y:/Projects/ToxvalModels/'
path = 'C:/Users/Administrator/OneDrive/Profile/Desktop/ToxvalModels/'

#%%
# Read the input data file
data_file = pd.read_excel(path+'data/toxval_pod_summary_human health_2019-08-20.xlsx', index_col = 'casrn')

## Read PubChem fingerprints and CDK descriptors generated in KNIME
pubchem = pd.read_csv(path+'data/toxval_pubchem.csv', index_col = 'row ID')
padel = pd.read_csv(path+'data/toxval_padel.csv', index_col = 'Name').dropna()
cdk = pd.read_csv(path+'data/toxval_cdk.csv', index_col = 'row ID').dropna()

#%%
## Select subsets of data based on studytype-species combinations

#specie = ['rat']
#specie = ['mouse']
#specie = ['rabbit']
#specie = ['rat', 'mouse']
specie = ['rat', 'mouse', 'rabbit']

#study = ['chronic', 'cancer']
#study = ['subchronic', 'repeat dose', 'short-term']
#study = ['reproductive','multigenerational reproductive']
#study = ['developmental', 'reproductive developmental']
#study = ['subacute']
study = ['chronic', 'cancer', 'subchronic', 'repeat dose', 'short-term', 'reproductive', 'multigenerational reproductive', 'developmental', 'reproductive developmental', 'subacute']

# Response data, Y, for developing QSAR models #'chronic', 'subchronic', 'reproductive', 'developmental, 'subacute'
Y_all = data_file[(data_file['risk_assessment_class'].isin(study)) \
               & (data_file['toxval_units'] == 'mg/kg-day') \
               & (data_file['toxval_type_supercategory'] == 'Point of Departure') \
               & (~data_file['toxval_type'].isin(['ERCL10','NOTEL', \
                  'EC0', 'EC5', 'EC10', 'EC20', 'EC50', 'EC90', 'EC100',\
                  'IC50', 'IC100',\
                  ])) \
               & (data_file['species_common'].isin(specie))][['toxval_numeric', 'species_common', 'risk_assessment_class']] 
               
#Y_all['specie'] = Y_all.apply(lambda col: pd.factorize(col)[0])['species_common'] #convert species to numeric

## Convert study type to number
Y_all.loc[Y_all['species_common']=='rat', 'specie'] = 0
Y_all.loc[Y_all['species_common']=='mouse', 'specie'] = 1
Y_all.loc[Y_all['species_common']=='rabbit', 'specie'] = 2

## Convert study type to number
Y_all.loc[Y_all['risk_assessment_class']=='chronic', 'study'] = 0
Y_all.loc[Y_all['risk_assessment_class']=='cancer', 'study'] = 0

Y_all.loc[Y_all['risk_assessment_class']=='subchronic', 'study'] = 1
Y_all.loc[Y_all['risk_assessment_class']=='repeat dose', 'study'] = 1
Y_all.loc[Y_all['risk_assessment_class']=='short-term', 'study'] = 1

Y_all.loc[Y_all['risk_assessment_class']=='reproductive', 'study'] = 2
Y_all.loc[Y_all['risk_assessment_class']=='multigenerational reproductive', 'study'] = 2

Y_all.loc[Y_all['risk_assessment_class']=='developmental', 'study'] = 3
Y_all.loc[Y_all['risk_assessment_class']=='reproductive developmental', 'study'] = 3

Y_all.loc[Y_all['risk_assessment_class']=='subacute', 'study'] = 4

## Log transform the values
Y_all['toxval_numeric'] = np.log10(Y_all['toxval_numeric'])

#%%
############################################################################################################
#------------------------------------------- Model 1 ------------------------------------------------------#
############################################################################################################
## single POD value for all study types and species
Y1 = Y_all.groupby(Y_all.index)['toxval_numeric'].median()

#------------------------------------------- Data Uncertainty ---------------------------------------------#
Y1_uncertainty = pd.DataFrame()
Y1_uncertainty['std'] = Y_all.groupby(Y_all.index)['toxval_numeric'].std()
Y1_uncertainty['mean'] = Y_all.groupby(Y_all.index)['toxval_numeric'].mean()
Y1_uncertainty['median'] = Y_all.groupby(Y_all.index)['toxval_numeric'].median()

#--------------------------------------Feature Selection---------------------------------------------------#
## Fingerprints and Descriptors

# Pubchem
X1_pubchem = pubchem.loc[Y1.index,:].dropna()
# Feature Selection on X1 fingerprints using 80% variance threshold
X1_pubchem = selectFeatures_VarThresh(X1_pubchem, 0.8)
# Remove culumns with >80% correlation
X1_pubchem = correlation(X1_pubchem, 0.80)

# Padel
X1_padel = padel.loc[Y1.index,:].dropna()
## Feature selection from Padel descriptors
X1_padel = normalizeDescriptors(X1_padel)
X1_padel = selectFeatures_RFE(X1_padel, Y1[X1_padel.index], 5)

# CDK
X1_cdk = cdk.loc[Y1.index,:].dropna()
## Feature selection from Padel descriptors
X1_cdk = normalizeDescriptors(X1_cdk)
X1_cdk = selectFeatures_RFE(X1_cdk, Y1[X1_cdk.index], 5)

#------------------------------------------Set the X and Y vector-------------------------------------------------#
X1 = pd.concat([X1_pubchem, X1_padel, X1_cdk], axis = 1).dropna() # species, r_a_c
# Update Y based on X columns that are retained
Y1 = Y1[X1.index]

#############################################################################################################
##----------------------------------------Save Descriptors for the model ------------------------------------------- ----------#
#############################################################################################################
## Output file to capture the descriptors in the model for external predictions
features = pd.DataFrame({'Fingerprints': [X1_pubchem.columns.values.tolist()], 'PaDEL': [X1_padel.columns.values.tolist()], 'CDK': [X1_cdk.columns.values.tolist()]})
features.to_csv(path+'output/FinalModelFeatures.csv')

#%%
#############################################################################################################
##------------------------------------------- Data for Model 2 ------------------------------------------- ----------#
#############################################################################################################
Y2 = Y_all.groupby([Y_all.index, 'study', 'specie'])['toxval_numeric'].median().reset_index()
Y2.index = Y2.casrn

#------------------------------------------- Data Uncertainty ---------------------------------------------#
Y2_uncertainty = pd.DataFrame()
Y2_uncertainty['std'] = Y_all.groupby([Y_all.index, 'study', 'specie'])['toxval_numeric'].std()
Y2_uncertainty['mean'] = Y_all.groupby([Y_all.index, 'study', 'specie'])['toxval_numeric'].mean()
Y2_uncertainty['median'] = Y_all.groupby([Y_all.index, 'study', 'specie'])['toxval_numeric'].median()
Y2_uncertainty.reset_index(inplace=True)
Y2_uncertainty.index = Y2_uncertainty.casrn

#%%
#------------------Data to model
datatomodel = pd.concat([X1_pubchem, X1_padel, X1_cdk], axis = 1).join(Y2).dropna() # species, r_a_c
datatomodel_train, datatomodel_test = train_test_split(datatomodel, test_size = 0.20, random_state = 40)

## Model 2: Set data for bootstrapping
X_train = datatomodel_train[datatomodel_train.columns.difference(['index','toxval_numeric','casrn', 'std', 'species_common', 'risk_assessment_class'])]
X_test = datatomodel_test[datatomodel_test.columns.difference(['index','toxval_numeric','casrn', 'std', 'species_common', 'risk_assessment_class'])]
Y_train = datatomodel_train['toxval_numeric']
Y_test = datatomodel_test['toxval_numeric']

n_bootstraps = 100
# Create bootstrap values for each chemical
train_bootstraps = distribution_samples(datatomodel_train[['study', 'specie', 'toxval_numeric']], 0.5, 40, n_bootstraps)
# Combine the bootstraps 
datatomodel_train_bootstraps = pd.concat([datatomodel_train, train_bootstraps], axis = 1)


#############################################################################################################
##------------------------------------------- Build Models ------------------------------------------- ----------#
#############################################################################################################
Y_train_predicted = pd.DataFrame(index=X_train.index)
Y_test_predicted = pd.DataFrame(index=X_test.index)

Y_train_predicted_byChem = pd.DataFrame(index=X_train.index.unique())
Y_test_predicted_byChem = pd.DataFrame(index=X_test.index.unique())

metrics = pd.DataFrame()#columns=['RMSE_train', 'RMSE_test', 'R2_train', 'R2_TEST', 'MAD_train', 'MAD_test'])
#%%
for n in range(0,100):
    # Time at start of each boostrap run
    t0 = time.time()
    
    for idx in X_train:
        Y_train_bootstrap = datatomodel_train_bootstraps['bootstrap_%d'%n] 
 
    n_fold = 5
    ## Train the model and calculate errors using n_fold cross validation
    clf = RandomForestRegressor(bootstrap=True, criterion='mse', max_depth=None,\
           max_features='sqrt', max_leaf_nodes=None,\
           min_impurity_decrease=0.0, min_impurity_split=None,\
           min_samples_leaf=1, min_samples_split=2,\
           min_weight_fraction_leaf=0.0, n_estimators=750, n_jobs=None,\
           oob_score=False, random_state=5, verbose=0, warm_start=False)

    pkl_filename = path+'output/FinalBootstrapModels/model_bootstrap_%d.pkl'%n
    with open(pkl_filename, 'wb') as file:
        pickle.dump(clf, file)
 
    ##  Cross-validation on training set
    Y_train_predicted['specie'] =  X_train['specie']
    Y_train_predicted['study'] =  X_train['study']
    Y_train_predicted['bootstrap_%d'%n] = predict_y(clf, X_train, Y_train_bootstrap, n_fold)

    metrics.loc['bootstrap_%d'%n,'RMSE_train'] = np.sqrt(sm.mean_squared_error(Y_train_bootstrap, Y_train_predicted['bootstrap_%d'%n]))
    metrics.loc['bootstrap_%d'%n,'R2_train'] = r2_score(Y_train_bootstrap, Y_train_predicted['bootstrap_%d'%n])

    ## External validation on test set
    Y_test_predicted['specie'] =  X_test['specie']
    Y_test_predicted['study'] =  X_test['study']
    clf = clf.fit(X_train, Y_train_bootstrap)
    Y_test_predicted['bootstrap_%d'%n]  = clf.predict(X_test)
    
#    pkl_filename = path+'output/FinalBootstrapModels/model_bootstrap_%d.pkl'%n
#    with open(pkl_filename, 'wb') as file:
#        pickle.dump(clf, file)
#    #Y_test_predicted['bootstrap_%d'%n]  = predict_test_y(clf, X_train, Y_train_bootstrap, X_test)

    metrics.loc['bootstrap_%d'%n,'RMSE_test'] = np.sqrt(sm.mean_squared_error(Y_test, Y_test_predicted['bootstrap_%d'%n]))
    metrics.loc['bootstrap_%d'%n,'R2_test'] = r2_score(Y_test, Y_test_predicted['bootstrap_%d'%n])

    # Time at end of each boostrap run; time elapsed
    t1 = time.time()
    total_time = t1-t0
    print("Bootstrap %d. Total time elapsed: %f" %(n, total_time))
    
#%%
#############################################################################################################
##----------------------------- Calculate Summary Metrics --------------------------------------------------#
#############################################################################################################
summary_metrics = pd.Series()
summary_metrics['Mean RMSE Train (Bootstrap)'] = metrics['RMSE_train'].mean()
summary_metrics['Mean R2 Train (Bootstrap)'] = metrics['R2_train'].mean()
summary_metrics['Mean RMSE Test (Bootstrap)'] = metrics['RMSE_test'].mean()
summary_metrics['Mean R2 Test (Bootstrap)'] = metrics['R2_test'].mean()
#%%

#############################################################################################################
##----------------------------- Calculate Summary Predictions------------------------------------ ----------#
#############################################################################################################
#-- Summary chemical level predictions per chemical overall
## train
Y_train_predicted_byChem = Y_train_predicted.groupby([Y_train_predicted.index]).median()
Y_train_byChem = Y_train.groupby([Y_train.index]).median()

subset = Y_train_predicted_byChem.copy()
subset = subset.loc[:,subset.columns.str.startswith('b')]
Y_train_predicted_byChem['Mean Prediction (Train)'] = subset.mean(axis=1)
Y_train_predicted_byChem['Median Prediction (Train)'] = subset.median(axis=1)
Y_train_predicted_byChem['Std Prediction (Train)'] = subset.std(axis=1)
#%%
## test
Y_test_predicted_byChem = Y_test_predicted.groupby([Y_test_predicted.index]).median()
Y_test_byChem = Y_test.groupby([Y_test.index]).median()
subset = Y_test_predicted_byChem.copy()
subset = subset.loc[:,subset.columns.str.startswith('b')]
Y_test_predicted_byChem['Mean Prediction (Test)'] = subset.mean(axis=1)
Y_test_predicted_byChem['Median Prediction (Test)'] = subset.median(axis=1)
Y_test_predicted_byChem['Std Prediction (Test)'] = subset.std(axis=1)

#%%
#--Summary metrics per chemical overall
summary_metrics['RMSE Train (byChem)'] = np.sqrt(sm.mean_squared_error(Y_train_byChem, Y_train_predicted_byChem['Mean Prediction (Train)']))
summary_metrics['R2 Train (byChem)'] = r2_score(Y_train_byChem, Y_train_predicted_byChem['Mean Prediction (Train)'])
#%%
summary_metrics['RMSE Test (byChem)'] = np.sqrt(sm.mean_squared_error(Y_test_byChem, Y_test_predicted_byChem['Mean Prediction (Test)']))
summary_metrics['R2 Test (byChem)'] = r2_score(Y_test_byChem, Y_test_predicted_byChem['Mean Prediction (Test)'])


#%%
#############################################################################################################
##------------------------- Plot true vs predicted for each classifier (training)---------------------------#
#############################################################################################################
rindex =  np.array(random.sample(Y_train_predicted_byChem.index.tolist(), 50))
## set yerror
yerror = Y_train_predicted_byChem.loc[rindex,'Std Prediction (Train)']
yerror.replace(np.nan, 0, inplace=True)
## set x error
xerror = Y2_uncertainty.groupby(Y2_uncertainty.index).mean().loc[rindex,'std']
xerror.replace(np.nan, 0, inplace=True)
 
## set x, y
x = Y_train_byChem.loc[rindex]
y = Y_train_predicted_byChem.loc[rindex,'Median Prediction (Train)']
## plot
ax = plt.figure(figsize=(8,6), dpi = 200)
plt.errorbar(x = x, \
             y = y, yerr = yerror, \
             fmt = 'o', ecolor = 'r', color = 'r', markersize='3', alpha=0.75, label = 'Predicted Error')
plt.errorbar(x = x, xerr = xerror, \
             y = y,\
             fmt = 'o', ecolor = 'k', color = 'g', markersize='3', alpha=0.75, label = 'Observed Error')


#ax.text(0.15, 0.80, r'Mean $RMSE (Bootstrap): %.2f$' % summary_metrics['Mean RMSE Train (Bootstrap)'] , fontsize = 20)
#ax.text(0.15, 0.75, r'Mean $R^{2} (Bootstrap): %.2f$' % summary_metrics['Mean R2 Train (Bootstrap)'] , fontsize = 20)

ax.text(0.15, 0.80, r'$RMSE: %.2f$' % summary_metrics['RMSE Train (byChem)'], fontsize = 20)
ax.text(0.15, 0.75, r'$R^{2}: %.2f$' % summary_metrics['R2 Train (byChem)'], fontsize = 20)

## plot the error lines
plt.plot([-2, 6], [-2, 6], c='k')
plt.plot([-2, 5],[-1, 6],'b', linestyle = '--')
plt.plot([-2, 5],[-3, 4],'b', label = '$\pm$1(log10) error interval', linestyle = '--')

plt.legend(loc='lower right', numpoints = 1, scatterpoints = 1, fontsize = 12)
plt.xlim([-2, 5])
plt.ylim([-2, 5])
plt.xticks(fontsize = 28, rotation = 90)
plt.yticks(fontsize = 28)
plt.xlabel('Observed', size = 26, labelpad = 18)
plt.ylabel('Predicted', size = 26, labelpad = 18) 

plt.savefig(path+'output/100Bootsrap-TrainCI50', bbox_inches='tight')  

#%%
#Y_train_predicted.index = Y_train_predicted['Unnamed: 0']
#Y_train_predicted.drop('Unnamed: 0', axis = 1, inplace=True)
#
#Y_test_predicted.index=Y_test_predicted['Unnamed: 0']
#Y_test_predicted.drop('Unnamed: 0', axis = 1, inplace=True)
#
##%%
#Y_train_predicted = pd.read_csv(path+'output/Bootstrap_Y_train_predicted.csv')
#
#Y_test_predicted = pd.read_csv(path+'output/Bootstrap_Y_test_predicted.csv')
#Y_test_predicted.index=Y_test_predicted['Unnamed: 0']
#Y_test_predicted.drop('Unnamed: 0', axis = 1, inplace=True)
#
#Y_train_predicted_byChem = pd.read_csv(path+'output/Bootstrap_Y_train_predicted_byChem.csv')
#
#Y_test_predicted_byChem = pd.read_csv(path+'output/Bootstrap_Y_test_predicted_byChem.csv')
#Y_test_predicted_byChem.index=Y_test_predicted_byChem['Unnamed: 0']
#Y_test_predicted_byChem.drop('Unnamed: 0', axis = 1, inplace=True)
#
#summary_metrics = pd.read_csv(path+'output/Bootstrap_summary_metrics.csv', index_col = 'Metric')

#%%
#############################################################################################################
##------------------------- Plot true vs predicted for each classifier (test)---------------------------#
#############################################################################################################
random.seed(a=0)
rindex =  np.array(random.sample(Y_test_predicted_byChem.index.tolist(), 50))
## set yerror
yerror = 2*Y_test_predicted_byChem.loc[rindex, 'Std Prediction (Test)']
yerror.replace(np.nan, 0, inplace=True)
## set x error
xerror = 2*Y2_uncertainty.groupby(Y2_uncertainty.index).mean().loc[rindex,'std']
#xerror = 1.96*Y1_uncertainty.loc[rindex,'std']
xerror.replace(np.nan, 0, inplace=True)

## set x, y
x = Y_test_byChem.loc[rindex]
y = Y_test_predicted_byChem.loc[rindex,'Median Prediction (Test)']
## plot
ax = plt.figure(figsize=(8,6), dpi = 200)
plt.errorbar(x = x, \
             y = y, yerr = yerror, \
             fmt = 'o', ecolor = 'r', color = 'r', markersize='3', alpha=0.75, label = 'Predicted Error')
plt.errorbar(x = x, xerr = xerror, \
             y = y,\
             fmt = 'o', ecolor = 'k', color = 'g', markersize='3', alpha=0.75, label = 'Observed Error')


ax.text(0.15, 0.80, r'$RMSE: %.2f$' % summary_metrics.loc['RMSE Test (byChem)'], fontsize = 20)
ax.text(0.15, 0.75, r'$R^{2}: %.2f$' % summary_metrics.loc['R2 Test (byChem)'], fontsize = 20)

## plot the error lines
plt.plot([-2, 6], [-2, 6], c='k')
plt.plot([-2, 5],[-1, 6],'b', linestyle = '--')
plt.plot([-2, 5],[-3, 4],'b', label = '$\pm$1(log10) error interval', linestyle = '--')

plt.legend(loc='lower right', numpoints = 1, scatterpoints = 1, fontsize = 12)
plt.xlim([-2, 5])
plt.ylim([-2, 5])
plt.xticks(fontsize = 28, rotation = 90)
plt.yticks(fontsize = 28)
plt.xlabel('Observed', size = 26, labelpad = 18)
plt.ylabel('Predicted', size = 26, labelpad = 18) 

plt.savefig(path+'output/100Bootsrap-TestCI50-2', bbox_inches='tight')  


#%%
#############################################################################################################
##----------------------------- Save all files to disk ------------------------------------ ----------#
#############################################################################################################

## re-convert study type and species to number 
# training
Y_train_predicted.loc[Y_train_predicted['specie']==0, 'specie'] = 'rat'
Y_train_predicted.loc[Y_train_predicted['specie']==1, 'specie'] = 'mouse'
Y_train_predicted.loc[Y_train_predicted['specie']==2, 'specie'] = 'rabbit'

Y_train_predicted.loc[Y_train_predicted['study']== 0, 'study'] = 'CHR'
Y_train_predicted.loc[Y_train_predicted['study']==1, 'study'] = 'SUBCHR'
Y_train_predicted.loc[Y_train_predicted['study']==2, 'study'] = 'REP'
Y_train_predicted.loc[Y_train_predicted['study']==3, 'study'] = 'DEV'
Y_train_predicted.loc[Y_train_predicted['study']==4, 'study'] = 'SAC'

# test
Y_test_predicted.loc[Y_test_predicted['specie']==0, 'specie'] = 'rat'
Y_test_predicted.loc[Y_test_predicted['specie']==1, 'specie'] = 'mouse'
Y_test_predicted.loc[Y_test_predicted['specie']==2, 'specie'] = 'rabbit'

Y_test_predicted.loc[Y_test_predicted['study']== 0, 'study'] = 'CHR'
Y_test_predicted.loc[Y_test_predicted['study']==1, 'study'] = 'SUBCHR'
Y_test_predicted.loc[Y_test_predicted['study']==2, 'study'] = 'REP'
Y_test_predicted.loc[Y_test_predicted['study']==3, 'study'] = 'DEV'
Y_test_predicted.loc[Y_test_predicted['study']==4, 'study'] = 'SAC'

## re-convert study type and species to number by chem files
# training
Y_train_predicted_byChem.loc[Y_train_predicted_byChem['specie']==0, 'specie'] = 'rat'
Y_train_predicted_byChem.loc[Y_train_predicted_byChem['specie']==1, 'specie'] = 'mouse'
Y_train_predicted_byChem.loc[Y_train_predicted_byChem['specie']==2, 'specie'] = 'rabbit'

Y_train_predicted_byChem.loc[Y_train_predicted_byChem['study']== 0, 'study'] = 'CHR'
Y_train_predicted_byChem.loc[Y_train_predicted_byChem['study']==1, 'study'] = 'SUBCHR'
Y_train_predicted_byChem.loc[Y_train_predicted_byChem['study']==2, 'study'] = 'REP'
Y_train_predicted_byChem.loc[Y_train_predicted_byChem['study']==3, 'study'] = 'DEV'
Y_train_predicted_byChem.loc[Y_train_predicted_byChem['study']==4, 'study'] = 'SAC'

# test
Y_test_predicted_byChem.loc[Y_test_predicted_byChem['specie']==0, 'specie'] = 'rat'
Y_test_predicted_byChem.loc[Y_test_predicted_byChem['specie']==1, 'specie'] = 'mouse'
Y_test_predicted_byChem.loc[Y_test_predicted_byChem['specie']==2, 'specie'] = 'rabbit'

Y_test_predicted_byChem.loc[Y_test_predicted_byChem['study']== 0, 'study'] = 'CHR'
Y_test_predicted_byChem.loc[Y_test_predicted_byChem['study']==1, 'study'] = 'SUBCHR'
Y_test_predicted_byChem.loc[Y_test_predicted_byChem['study']==2, 'study'] = 'REP'
Y_test_predicted_byChem.loc[Y_test_predicted_byChem['study']==3, 'study'] = 'DEV'
Y_test_predicted_byChem.loc[Y_test_predicted_byChem['study']==4, 'study'] = 'SAC'

Y_train_predicted.to_csv(path+'output/Bootstrap_Y_train_predicted.csv')
Y_test_predicted.to_csv(path+'output/Bootstrap_Y_test_predicted.csv')
Y_train_predicted_byChem.to_csv(path+'output/Bootstrap_Y_train_predicted_byChem.csv')
Y_test_predicted_byChem.to_csv(path+'output/Bootstrap_Y_test_predicted_byChem.csv')
summary_metrics.to_csv(path+'output/Bootstrap_summary_metrics.csv')
   
#%%        
### Example POD distribution bisphenol A - DTXCID30182
import matplotlib.mlab as mlab
median = Y_all.loc['80-05-7','toxval_numeric'].median()

ax = plt.figure(figsize=(8,6), dpi = 200)
mu = median
sigma = 0.5
x = np.linspace(mu - 3*sigma, mu + 3*sigma, 100)
plt.plot(x, mlab.normpdf(x, mu, sigma), c= 'g')
plt.ylim([0,1])

ax.text(0.15, 0.80, r'$\mu: %.2f$' % mu, fontsize = 24)
ax.text(0.15, 0.70, r'$\sigma: %.2f$' % sigma, fontsize = 24)

plt.savefig(path+'output/boostrapEx.png', bbox_inches='tight')  

